ReadCondition

Struct ReadCondition 

pub struct ReadCondition { /* private fields */ }
ReadCondition — Spec 2.2.2.5.8 / 2.2.4.5 Trigger State.

Eine ReadCondition ist an einen DataReader gebunden und triggert true, wenn der Reader Samples enthaelt, die in alle drei Masks (sample_state_mask, view_state_mask, instance_state_mask) passen.

Design: Die Trigger-Logik selbst (das eigentliche “hat der Reader Samples mit diesen Masks?”-Query) ist vom Caller als Closure injiziert, weil der DataReader-Sample-Cache nicht objekt-safe gequeried werden kann ohne weitere Infrastructure- Aenderungen. Der DCPS-API-Konsument (idR DataReader::create_readcondition) liefert die Closure in Form (sm, vm, im) -> bool.

Implementations

Source

impl ReadCondition

pub fn new<F>( sample_state_mask: u32, view_state_mask: u32, instance_state_mask: u32, trigger: F, ) -> Arc<Self>
where F: Fn(u32, u32, u32) -> bool + Send + Sync + 'static,

Konstruktor mit Trigger-Closure.

pub fn get_sample_state_mask(&self) -> u32

Spec 2.2.2.5.8 get_sample_state_mask.

pub fn get_view_state_mask(&self) -> u32

Spec 2.2.2.5.8 get_view_state_mask.

pub fn get_instance_state_mask(&self) -> u32

Spec 2.2.2.5.8 get_instance_state_mask.