TopicListener

Trait TopicListener 

pub trait TopicListener: Send + Sync {
    // Provided method
    fn on_inconsistent_topic(
        &self,
        _topic: InstanceHandle,
        _status: InconsistentTopicStatus,
    ) { ... }
}
TopicListener — Spec 2.2.2.3.2 + 2.2.4.2.5.

Exactly one callback: on_inconsistent_topic. The topic parameter is passed as an opaque InstanceHandle (Spec 2.3.3 IDL-PSM).

Provided Methods

fn on_inconsistent_topic( &self, _topic: InstanceHandle, _status: InconsistentTopicStatus, )

Spec 2.2.4.2.5 — called when another topic with the same name but a different type definition is discovered.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors