RequestedDeadlineMissedStatus

Struct RequestedDeadlineMissedStatus 

pub struct RequestedDeadlineMissedStatus {
    pub total_count: i32,
    pub total_count_change: i32,
    pub last_instance_handle: InstanceHandle,
}
REQUESTED_DEADLINE_MISSED_STATUS — Spec 2.2.4.1 + 2.2.2.5.6.

Reader-Seite. Counter, wie oft der Reader keine Sample innerhalb des requested DEADLINE bekommen hat.

Fields

total_count: i32

Total cumulative count of requested-deadline misses.

total_count_change: i32

Increment since last read.

last_instance_handle: InstanceHandle

Handle of the last instance for which the deadline was missed.