SampleRejectedStatus

Struct SampleRejectedStatus 

pub struct SampleRejectedStatus {
    pub total_count: i32,
    pub total_count_change: i32,
    pub last_reason: SampleRejectedStatusKind,
    pub last_instance_handle: InstanceHandle,
}
SAMPLE_REJECTED_STATUS — Spec 2.2.4.1 + 2.2.2.5.6.

Raised when the reader has dropped a sample due to a RESOURCE_LIMITS violation.

Fields

total_count: i32

Total cumulative count of rejected samples.

total_count_change: i32

Increment since last read.

last_reason: SampleRejectedStatusKind

Reason for the most recent rejection.

last_instance_handle: InstanceHandle

Handle of the instance that was the target of the most recent rejection.