pub enum SampleStateKind {
NotRead,
Read,
}Expand description
SampleStateKind (DDS 1.4 2.2.2.5.1.1) — maintained per reader.
Variants
NotRead
Sample has never been delivered to the application via
read/take.
Read
Sample has already been delivered at least once via read
(after take it is gone, so this only matters for read).
Implementations
Sourceimpl SampleStateKind
impl SampleStateKind
Sourcepub const fn is_not_read(&self) -> bool
pub const fn is_not_read(&self) -> bool
true if this is an as-yet-unread sample.