pub struct LivelinessChangedStatus {
pub alive_count: i32,
pub not_alive_count: i32,
pub alive_count_change: i32,
pub not_alive_count_change: i32,
pub last_publication_handle: InstanceHandle,
}Expand description
LIVELINESS_CHANGED_STATUS — Spec 2.2.4.1 + 2.2.2.5.6.
Reader-Seite: “Reports the status of the liveliness of one or more
DataWriter objects that are matched with the DataReader.”
Anders als die Plain-Counter-Statuses dürfen *_count_change hier
negativ werden, wenn z.B. ein als “alive” deklarierter Writer
jetzt als “not_alive” zählt (übersprungen von alive_count zu
not_alive_count).
Fields
alive_count: i32Number of currently-alive matched writers.
not_alive_count: i32Number of currently-not-alive matched writers.
alive_count_change: i32Change in alive_count since the last read.
not_alive_count_change: i32Change in not_alive_count since the last read.
last_publication_handle: InstanceHandleHandle of the last writer that triggered the change.