pub struct SampleRejectedStatus {
pub total_count: i32,
pub total_count_change: i32,
pub last_reason: SampleRejectedStatusKind,
pub last_instance_handle: InstanceHandle,
}Expand description
SAMPLE_REJECTED_STATUS — Spec 2.2.4.1 + 2.2.2.5.6.
Wird ausgelöst, wenn der Reader ein Sample wegen einer RESOURCE_LIMITS-Verletzung verworfen hat.
Fields
total_count: i32Total cumulative count of rejected samples.
total_count_change: i32Increment since last read.
last_reason: SampleRejectedStatusKindReason for the most recent rejection.
last_instance_handle: InstanceHandleHandle of the instance that was the target of the most recent rejection.